;**********************************************************************
;    NOM:   PREAMP93    rs232 + MUTE + regl VOLin avec MUTE + vol+/-  *
;    Date:  13/06/2002                                                *
;    Version:   amelioration GESTION CLAVIER, mode FADER/VOLIN        *
;    Circuit:   preampli, ampli, commutateur                          *
;    Auteur:                                                          *
;     FADER : MUTE(6 secondes) ; OFF_FADER : MUTE (6s)                *
;     VOLIN : MUTE + Vol- (6 sec.) ; OFF_VOLIN : MUTE (6s)            *
;    REGLAGE DANS EEPROM : lecture + ecriture                         *
;	<<<< Manque reglage canal sur RS232, lecture param(EEPROM)>>>>*
;       <<<< Tester canal3 LM1972 (pour regler canal1+2 simultanement)*
;**********************************************************************
	LIST      p=16F84a            ; Dfinition de processeur
	#include <p16F84a.inc>        ; Dfinitions des constantes
	ERRORLEVEL -302
	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _HS_OSC

;*********************************************************************
;                              ASSIGNATIONS                          *
;*********************************************************************
OPTIONVAL	EQU	B'00000000' ; pull-up ON, failling RB0/INT
;EQU	H'0040'	; Valeur registre option
                                ; Rsistance pull-up ON<--**
                                ; Interrupt flanc montant RB0
                                ; Prscaler timer  2 (exemple)

INTERMASK	EQU	B'10010000' ; GIE, RB0	
;H'0090' ; Masque d'interruption
;                               ; Interruptions sur RB0 (exemple)


;*********************************************************************
;                             DEFINE                                 *
;*********************************************************************
#DEFINE LOAD	PORTA,2		; LOAD/SHIFT
#DEFINE CLK	PORTA,1		; ligne de clk
#DEFINE DIN	PORTA,3		; DATA IN

#DEFINE UP	PORTB,6		; vol+
#DEFINE DOWN	PORTB,7		; vol-
#DEFINE Mute	PORTB,3		; mute

#DEFINE Sp	PORTB,4		; source+
#DEFINE Sm	PORTB,5		; source-
#DEFINE DemuxA	PORTA,0		; DemuxA 4051
#DEFINE DemuxB	PORTB,1		; DemuxB 4051
#DEFINE DemuxC	PORTB,2		; DemuxC 4051

#DEFINE RX232	PORTB,0		; port serie
#DEFINE TX232	PORTA,4		; idem  + PULL-UP (a rajouter sur circuit : br1-14 :74HC14)
;--------------------------
; Debit  | 1 bit | 1/2 bit|
;--------------------------
;  9600  |  18h  |   0Ch  | 
;  4800  |  32h  |   19h  | 
;  2400  |  66h  |   33h  | 
;  1200  |  CEh  |   67h  |
#DEFINE	BAUDS_1	h'18' 
#DEFINE	BAUDS_2	h'0C'
; 9600,N,8,1 : 9600bds, pas de parite, 8 bits data, 1 bit de stop

#DEFINE	VOLdepart	H'66'		; -xxdB 2x20W
#DEFINE VOLdepartAV	H'76'		;       2x70W
#DEFINE	SourceMAX	H'05'		; 5 sources maxi. (8)

;----- Flag Touches --------------------------
;0, 0, 0, UP, DOWN, Mute, Sp, Sm
;7  6  5   4     3     2   1   0 : bit,touche
#DEFINE	tAUCUNE	7
#DEFINE	tFADER	6
#DEFINE	tVOLIN	5
#DEFINE	tUP	4
#DEFINE	tDOWN	3
#DEFINE	tMute	2
#DEFINE	tSp	1
#DEFINE	tSm	0
;*********************************************************************
;                             MACRO                                  *
;*********************************************************************
BANK0	macro
	bcf	STATUS , RP0	; passer banque0
	endm

BANK1	macro
	bsf	STATUS , RP0	; passer banque1
	endm

READEE	macro	adeeprom	; lecture d'une adresse EEPROM
	movf	adeeprom,w	; charger adresse eeprom
	movwf	EEADR		; adresse  lire dans registre EEADR
	bsf	STATUS , RP0	; passer en banque1
	bsf	EECON1 , RD	; lancer la lecture EEPROM
	bcf	STATUS , RP0	; repasser en banque0
	movf	EEDATA , w	; charger valeur lue dans W
	endm			; fin de la macro

WRITEE	macro	addwrite,data	; la donne se trouve dans data
;	LOCAL	loop		; tiquette locale
	local	wait		;#debug
	BANK1
wait
	btfsc	EECON1,WR	; teste la fin d'ecriture (10ms)
	goto	wait
	BANK0
	movf	data,w		; charger donnee
	movwf	EEDATA		; placer data dans registre
	movf	addwrite,w	; charger adresse d'criture
	movwf	EEADR		; placer dans registre
;loop
	bcf	INTCON , GIE	; interdire interruptions
;	btfsc	INTCON , GIE	; tester si GIE bien  0
;	goto	loop		; non, recommencer
	bsf	STATUS , RP0	; passer en banque1
	bcf	EECON1 , EEIF	; effacer flag de fin d'criture
	bsf	EECON1 , WREN	; autoriser accs criture
	movlw	0x55		; charger 0x55
	movwf	EECON2		; envoyer commande
	movlw	0xAA		; charger 0xAA
	movwf	EECON2		; envoyer commande
	bsf	EECON1 , WR	; lancer cycle d'criture
	bcf	EECON1 , WREN	; verrouiller prochaine criture
	bsf	INTCON , GIE	; rautoriser interruptions
	bcf	STATUS , RP0	; repasser en banque0
	endm

LECT_PARAM macro	Mvolume, MvolIN, MvolumeAV, canal, adrTemp, cpt
			; byte , byte  , byte  , const byte, tmp byte*2
	local	LECT_PARAM1
	local	LECT_PARAM2
	movf	canal,w
	movwf	cpt		; cpt = canal
	incf	cpt,f		; cpt = canal+1
	clrf	adrTemp
	goto	LECT_PARAM2
LECT_PARAM1			; canal = canal*3
	movlw	d'3'
	addwf	adrTemp,f	; adresse EEPROM de base
LECT_PARAM2
	decfsz	cpt,f
	goto	LECT_PARAM1
	READEE	adrTemp		; adresse base -> volume
	movwf	Mvolume
	incf	adrTemp,f
	READEE	adrTemp		; adresse base+1 -> volIN
	movwf	MvolIN
	incf	adrTemp,f
	READEE	adrTemp		; adresse base+2 -> volumeAV
	movwf	MvolumeAV
	endm


ECR_PARAM macro  	Mvolume, MvolIN, MvolumeAV, canal, adrTemp, cpt
			;cst byte,cst byte,cst byte,cst byte, tmp byte*2
	local	ECR_PARAM1
	local	ECR_PARAM2
	movf	canal,w
	movwf	cpt		; cpt = canal
	incf	cpt,f		; cpt = canal+1
	clrf	adrTemp
	goto	ECR_PARAM2
ECR_PARAM1			; canal = canal*3
	movlw	d'3'
	addwf	adrTemp,f	; adresse EEPROM de base
ECR_PARAM2
	decfsz	cpt,f
	goto	ECR_PARAM1	
	WRITEE	adrTemp,Mvolume	; volume -> adresse base 
	incf	adrTemp,f
	WRITEE	adrTemp,MvolIN	; volIN -> adresse base+1 
	incf	adrTemp,f
	WRITEE	adrTemp,MvolumeAV; volumeAV -> adresse base+2
	endm

CRLF	macro
	movlw	'\n'	;H'0A'
	call	TX_RS
	movlw	'\r'	;H'0D'
	call	TX_RS
	endm

ESP	macro
	movlw	' '
	call	TX_RS
	endm

;=====================================================================
;	REGLAGE Source RS232
;=====================================================================
SWITCH_SOURCE232 macro
	;;call	ECR_PARAMETRES	; sauvegarde param. sources actuelle
	movf	RB0source,w
	movwf	source		; nouvelle source
	call	SWITCH_SOURCE	; commutation
	endm

;*********************************************************************
;                   DECLARATIONS DE VARIABLES                        *
;*********************************************************************
	CBLOCK 0x00C   		; dbut de la zone variables
	volume			; niveau du volume Arriere	
	volumeAV		; volume Avant
	volIN			; attenuation d'entree
	source			; source 0 a 5 ou 6
	touche			; numero de la derniere touche appuyee
	CPTtouche		; compteur du nombre d'appuis sur une touche
	
	data1			; donnee LM1972
	tmp			; sauvegarde temporaire Volume (+Attenuation)
	boucle			; compteur de boucle LM1972
	address			; adresse LM1972
	
	tempo1			; temporisation
	tempo2			; temporisation

	cpt1			; compteur nbre de bit RS232
	cpt2	 		; compteur tempo d'une duree d'1 bit ou 1/2 bit
	buffer	 		; Stockage de la donnee a emettre RS232
	tampon		
	ascii1
	ascii2
	comp1

	w_temp			; Sauvegarde de W dans interruption
	status_temp 		; Sauvegarde de STATUS dans interrupt

	tmpEE1,tmpEE2		; ecriture/lecture EEPROM

	HEXA_Temp1,HEXA_TEMP	; conversion HEXA->ASCII
	HEXA_cpt1

	RB0tmp
	RB0rx
	RB0volumeAV,RB0volume,RB0volIN	; volumes temporaires
	RB0source

        ENDC			; Fin de la zone                        

;*********************************************************************
;                   DECLARATIONS DE LA ZONE EEPROM                   *
;*********************************************************************
	org	0x2100		; adresse debut zone eeprom
	;	Volume, VolIN, VolumeAV, ... *7
	;	2x20W	,input , 2x70W
	DE	VOLdepart, 0x00, VOLdepartAV	; source 0 PC
	DE	0x35, 0x00, 0x40		; source 1 Radio
	DE	0x70, 0x00, 0x25		; source 2 CD
	DE	0xFF, 0xFF, 0xFF		; source 3
	DE	0xFF, 0xFF, 0xFF		; source 4
	DE	0x00, 0x08, 0x00		; source 5
	DE	0xFF, 0xFF, 0xFF		; source 6

;**********************************************************************
;                      DEMARRAGE SUR RESET                            *
;**********************************************************************
	org	0x000 		; Adresse de dpart aprs reset
  	goto    init		; Adresse 0: initialiser

;**********************************************************************
;                     ROUTINE INTERRUPTION                            *
;**********************************************************************
			;sauvegarder registres	
			;---------------------
	org 	0x004		; adresse d'interruption
	movwf   w_temp  	; sauver registre W
	swapf	STATUS,w	; swap status avec rsultat dans w
	movwf	status_temp	; sauver status swapp

			; switch vers diffrentes interrupts
			; inverser ordre pour modifier priorits
			;----------------------------------------
	
;	btfsc	INTCON,T0IE	; tester si interrupt timer autorise
;	btfss	INTCON,T0IF	; oui, tester si interrupt timer en cours
;	goto 	intsw1		; non test suivant
;	call	inttimer	; oui, traiter interrupt timer
;	bcf	INTCON,T0IF	; effacer flag interrupt timer
;	goto	restorereg	; et fin d'interruption
				; SUPPRIMER CETTE LIGNE POUR
				; TRAITER PLUSIEURS INTERRUPT
				; EN 1 SEULE FOIS

intsw1
	btfsc	INTCON,INTE	; tester si interrupt RB0 autorise
	btfss	INTCON,INTF	; oui, tester si interrupt RB0 en cours
	goto 	intsw2		; non sauter au test suivant
	call	intrb0		; oui, traiter interrupt RB0
	bcf	INTCON,INTF	; effacer flag interupt RB0
	goto	restorereg	; et fin d'interruption
				; SUPPRIMER CETTE LIGNE POUR
				; TRAITER PLUSIEURS INTERRUPT
				; EN 1 SEULE FOIS

intsw2
;	btfsc	INTCON,RBIE	; tester si interrupt RB4/7 autorise
;	btfss	INTCON,RBIF	; oui, tester si interrupt RB4/7 en cours
;	goto 	intsw3		; non sauter
;	call	intrb4		; oui, traiter interrupt RB4/7
;	bcf	INTCON,RBIF	; effacer flag interupt RB4/7
;	goto	restorereg	; et fin d'interrupt

;intsw3

			;restaurer registres
			;-------------------
restorereg
	swapf	status_temp,w	; swap ancien status, rsultat dans w
	movwf   STATUS		; restaurer status
	swapf   w_temp,f	; Inversion L et H de l'ancien W
                       		; sans modifier Z
	swapf   w_temp,w  	; Rinversion de L et H dans W
				; W restaur sans modifier status
	retfie  		; return from interrupt

;**********************************************************************
;                     INTERRUPTION TIMER 0                            *
;**********************************************************************
inttimer
	return			; fin d'interruption timer
				; peut tre remplac par 
				; retlw pour retour code d'erreur

;**********************************************************************
;                     INTERRUPTION RB0/INT                            *
;**********************************************************************
;; intrb0 : cf plus haut MACRO
; tempo de 255*5us (1/9600=100us) avec test btfss RX232
; evite le blocage si detection 'parasite'
RX232stop 
;	local	RX232stop,RX232stop1,RX232stop2,RX232stop3
	clrf	tempo1
RX232stop1
	incfsz	tempo1,f
	goto	RX232stop2
	goto	RX232stop3
RX232stop2
	btfsc	RX232
	goto	RX232stop1
RX232stop3
	return

intrb0	
	call	RX_RS		; lecture RS232
	movlw	'R'	
	xorwf	buffer,w	; mode reglage
	btfsc	STATUS,Z	; si == 'R'
	goto	intrb0_10
	
	movlw	'S'
	xorwf	buffer,w	; mode source
	btfsc	STATUS,Z
	goto	intrbS1

	goto	intrb0_2	; fin si mauvais param ('R'||'S')
	
;;	call	RX232stop

intrb0_10
	call	RX232stop	
	call	RX_RS
	movf	buffer,w
	movwf	RB0volume
;;	call	TX_RS
	call	RX232stop

intrb0_11	
	call	RX_RS
	movf	buffer,w
	movwf	RB0volIN
;;	call	TX_RS
	call	RX232stop

intrb0_12
	call	RX_RS
	movf	buffer,w
	movwf	RB0volumeAV
;;	call	TX_RS
	call	RX232stop

;; Balise fin de transfert Reglage 'Z' 
	call	RX_RS		; lecture RS232
	movlw	'Z'	
	xorwf	buffer,w	; fin mode reglage
	btfss	STATUS,Z	; si == 'Z'
	goto	intrb0_2	; mauvaise balise -> retour

;; Nouveaux Parametres de volumes
	movf	RB0volumeAV,w
	movwf	volumeAV
	movf	RB0volume,w
	movwf	volume
	movf	RB0volIN,w
	movwf	volIN
	call	PREAMP
	goto	intrb0_3	; fin reception

intrbS1
	call	RX232stop
	call	RX_RS
	movf	buffer,w
	movwf	RB0source
;;	call	TX_RS
	call	RX232stop
;; Balise fin de transfert Reglage 'Z' 
	call	RX_RS		; lecture RS232
	movlw	'Z'	
	xorwf	buffer,w	; fin mode reglage
	btfss	STATUS,Z	; si == 'Z'
	goto	intrb0_2	; mauvaise balise -> retour

;; Nouvelle Source
	movf	RB0source,w
	movwf	source
	SWITCH_SOURCE232
	goto	intrb0_3	; fin reception

intrb0_2
	movlw	'B'		; Bad transfert
	call	TX_RS		; Erreur pendant le transfert
intrb0_3
	return			; fin d'interruption RB0/INT
				; peut tre remplac par 
				; retlw pour retour code d'erreur
;**********************************************************************
;                     INTERRUPTION RB4/RB7                            *
;**********************************************************************
intrb4
	return			; fin d'interruption RB0/RB4
				; peut tre remplac par 
				; retlw pour retour code d'erreur

;*********************************************************************
;                       INITIALISATIONS                              *
;*********************************************************************
init
	clrf	PORTA		; Sorties portA  0
	clrf	PORTB		; sorties portB  0
	BANK1			; passer banque1
	movlw	OPTIONVAL	; charger masque
	movwf	OPTION_REG	; initialiser registre option
 	clrf	EEADR		; permet de diminuer la consommation

	movlw	B'00000000'	; portA : TX232, DIN, LOAD, CLK, A
	movwf	TRISA		; '1' entree ; '0' sortie
	movlw	B'11111001'	; portB : Vol-, Vol+, Source-, Source+, Mute, C, B, RX232
	movwf	TRISB		; '1' entree ; '0' sortie
	BANK0			; passer banque0
	movlw	INTERMASK	; masque interruption
	movwf	INTCON		; charger interrupt control

;---------- parametres de depart -----------
;	movlw	VOLdepart	;
;	movwf	volume		; initialisation du volume
;	movlw	VOLdepartAV	;
;	movwf	volumeAV	; initialisation du volume Avant
;	movlw	b'00000000'	; 0dB IN
;	movwf	volIN		; initialisation du volIN
	clrf	source		; source = 0
	bsf	LOAD		; mise  1 LOAD
	LECT_PARAM volume, volIN, volumeAV, source, tmpEE1,tmpEE2;tmp, data1
	call	PREAMP		; initialisation attenuation + volume
	clrf	CPTtouche	; clavier : RAZ compteur de touche
	clrf	touche		; clavier : RAZ flag de la derniere touche
;--------- RS232 -------	
	bsf	TX232		;Init Tx RS232
	clrf 	tampon		;Efface le tampon clavier
;------------------------
	goto	start		; sauter programme principal

;*********************************************************************
;                         SOUS  PROGRAMMES                           *
;*********************************************************************
;=====================================================================
;	REGLAGE LM1972		a ameliorer (2 boucles identiques!!!)
;=====================================================================
LM1972		; volume dans W
	movwf	data1		; data = VOLUME a regler ; address = canal 0,1,2	
	bcf	CLK
	bcf	LOAD
	;bcf	PORTA,0	;LED0
	movlw	8
	movwf	boucle 		; boucle 8 fois
LM1972_addr
	rlf	address,f
	btfss	STATUS,C	; if bit de carry==0 : DIN=0
	bcf	DIN		
	btfsc	STATUS,C
	bsf	DIN		; 		else DIN=1
	bsf	CLK		; front montant CLK
	bcf	CLK		; RAZ CLK
	decfsz	boucle,f
	goto	LM1972_addr
	rlf	address,f	; derniere rotation pour remise en forme
	movlw	8
	movwf	boucle 		; boucle 8 fois
LM1972_data
	rlf	data1,f
	btfss	STATUS,C	; if bit de carry==0 : DIN=0
	bcf	DIN		
	btfsc	STATUS,C
	bsf	DIN		; 		else DIN=1
	bsf	CLK		; front montant CLK
	bcf	CLK		; RAZ CLK
	decfsz	boucle,f
	goto	LM1972_data
	rlf 	data1,f
	return

FIN1972	macro	
	bsf	LOAD		; fin du transfert
	bsf	CLK		;
	endm
;=====================================================================
;	TEMPORISATION	98ms
;=====================================================================
TEMPO			; 128*256*(3)=98ms
	movlw	d'128'
	movwf	tempo1
	;clrf	tempo1
TEMPO1
	clrf	tempo2
TEMPO2
	decfsz	tempo2,f
	goto	TEMPO2
	decfsz	tempo1,f
	goto	TEMPO1	
	return


;=====================================================================
;	TEMPORISATION	32ms
;=====================================================================
TEMPO_2			; 64*256*(3)=32ms (49)
	movlw	d'64'
	movwf	tempo1
	;clrf	tempo1
TEMPO1_2
	clrf	tempo2
TEMPO2_2
	decfsz	tempo2,f
	goto	TEMPO2_2
	decfsz	tempo1,f
	goto	TEMPO1_2	
	return

;=====================================================================
;	REGLAGE Attenuation + Volume
;		Volume -> volIN -> VolumeAV (registres a decalages)
;		1er       2nd      3eme    (prog inverse 3, 2, 1)
;=====================================================================
PREAMP
	bcf	INTCON,GIE	; bloquer interruptions
	movlw	b'00000000'	; canal1
	movwf	address
	movf	volumeAV,w	; charge volumeAV
	call	LM1972

	movlw	b'00000000'	; canal1
	movwf	address
	movf	volIN,w		; charge attenuation
	call	LM1972
	
	movlw	b'00000000'	; canal1
	movwf	address
	movf	volume,w	; charge volume
	call	LM1972
		
	FIN1972

	movlw	b'00000001'	; canal2
	movwf	address
	movf	volumeAV,w	; charge volumeAV
	call	LM1972

	movlw	b'00000001'	; canal2
	movwf	address
	movf	volIN,w		; charge attenuation
	call	LM1972
	
	movlw	b'00000001'	; canal2
	movwf	address
	movf	volume,w	; charge volume
	call	LM1972

	FIN1972			; fin du transfert
	bsf	INTCON,GIE	; autoriser interruptions
	return

;=====================================================================
;	REGLAGE Volume 					inutile !!!
;=====================================================================
PREAMP_VOL
	movlw	b'00000000'	; canal1
	movwf	address
	movf	volume,w	; charge volume
	call	LM1972
	FIN1972	

	movlw	b'00000001'	; canal2
	movwf	address
	movf	volume,w	; charge volume
	call	LM1972
	FIN1972

	;bsf	LOAD		; fin du transfert
	;bsf	CLK		;
	return

;=====================================================================
;	REGLAGE Volume + VolumeAV = MUTE
;=====================================================================
PREAMP_MUTE
	bcf	INTCON,GIE	; bloquer interruptions
	movlw	b'00000000'	; canal1
	movwf	address
	movlw	b'01111111'	; -100dB charge volumeAV
	call	LM1972

	movlw	b'00000000'	; canal1
	movwf	address
	movf	volIN,w		; charge attenuation
	call	LM1972
	
	movlw	b'00000000'	; canal1
	movwf	address
	movlw	b'01111111'	; -100dB
	call	LM1972
		
	FIN1972
	
	movlw	b'00000001'	; canal2
	movwf	address
	movlw	b'01111111'	; -100dB charge volumeAV
	call	LM1972

	movlw	b'00000001'	; canal2
	movwf	address
	movf	volIN,w		; charge attenuation
	call	LM1972
	
	movlw	b'00000001'	; canal2
	movwf	address
	movlw	b'01111111'	;-100dB
	call	LM1972

	FIN1972
	bsf	INTCON,GIE	; autoriser interruptions
	return
	
;=====================================================================
;	REGLAGE Source
;=====================================================================
SWITCH_SOURCE
	btfss	source,0	; bit 0
	bcf	DemuxA
	btfsc	source,0	; bit 0
	bsf	DemuxA
	btfss	source,1	; bit 1
	bcf	DemuxB
	btfsc	source,1	; bit 1
	bsf	DemuxB
	btfss	source,2	; bit 2
	bcf	DemuxC
	btfsc	source,2	; bit 2
	bsf	DemuxC
	LECT_PARAM volume, volIN, volumeAV, source, tmpEE1,tmpEE2;tmp, data1
	;;call	PREAMP
	return

;=====================================================================
;	LECTURE REGLAGES RS232 (source0  6)
;=====================================================================
	
;=====================================================================
;	Lecture des parametres en EEPROM
;=====================================================================
ECR_PARAMETRES
	ECR_PARAM volume, volIN, volumeAV, source, tmpEE1,tmpEE2;tmp, data1 ;;#debug
	return
	
;=====================================================================
;	Sous programme d'emission RS232 optimise
;=====================================================================
TX_RS
	movwf	buffer		; stocke donnee a transmettre
	movlw	0x08
	movwf	cpt1		; Init compteur nbre de bit
	bcf	TX232		; Bit de start
	call	tempo_bit	; Tempo 1 bit
tx1_seb
	rrf	buffer,f	; rotation donnee a transmettre
	btfss	STATUS,C	; Test si bit a emettre =1
	bcf	TX232
	btfsc	STATUS,C	; Test si=0
	bsf	TX232
	call	tempo_bit	; Tempo 1 bit
	decfsz	cpt1,f		; Decremente compteur nbre de bit
	goto 	tx1_seb
	rrf	buffer,f	; repositionne buffer
	bsf	TX232		; Bit de stop
	call	tempo_bit	; Tempo
	return
;=====================================================================
;	Sous programme de reception RS232
;=====================================================================
RX_RS
;;	btfsc	RX232		;Test si data in RS232 passe a "0" bit de start
;;	goto 	RX_RS
	;retlw	H'FF'		; erreur 1
	call	tempo_demi_bit	;Attente 1/2 bit
	btfsc	RX232		;Test si data in Rs232 toujours a "0"
	retlw	H'FF'			; erreur 2
	movlw	0x08		;Init compteur nbre de bit a acquerir
	movwf	cpt1
	clrf	buffer		;Raz buffer
	bcf	STATUS,C	;Carry=0
deb_bit
	call	tempo_bit	;Attente 1 bit
	btfsc	RX232		;Test si Din=0
	bsf	STATUS,C	;Carry=1
	rrf	buffer,f	;Rotation de buffer avec le carry
	decfsz	cpt1,f		;Test si cpt1=0
	goto 	deb_bit
	call	tempo_bit	;Attente 1 bit (bit de stop)
;;	call	tempo_demi_bit	;1/2 bit manquant
	retlw	0		;retour sans erreur
;	return
;=====================================================================
;	tempo d'une duree d'1 bit (RS232):
;=====================================================================
tempo_bit
	;movf	bauds_1,w
	movlw	BAUDS_1	
	movwf	cpt2
bd
	nop
	decfsz	cpt2,f
	goto 	bd
	return
	
;=====================================================================
;	tempo d'une duree d'1/2 bit (RS232):
;=====================================================================
tempo_demi_bit
	;movf	bauds_2,w
	movlw	BAUDS_2
	movwf	cpt2
dbd
	nop
	decfsz	cpt2,f
	goto 	dbd
	return
	
;=====================================================================
;	message 'TEST' (RS232):
;=====================================================================
TEST_232
	movlw	' '
	call	TX_RS
	movlw	'S'
	call	TX_RS
	movlw	'e'
	call	TX_RS
	movlw	'b'
	call	TX_RS
	return

;volume, volIN, volumeAV, source
SOURCE232
	movlw	'V'
	call	TX_RS
	movf	volume,w
	call	LCDHEXA		;TX_RS
	movlw	'V'
	call	TX_RS
	movlw	'i'
	call	TX_RS
	movf	volIN,w
	call	LCDHEXA		;TX_RS
	movlw	'V'
	call	TX_RS
	movlw	'a'
	call	TX_RS
	movlw	'v'
	call	TX_RS
	movf	volumeAV,w
	call	LCDHEXA		;TX_RS
	movlw	'S'
	call	TX_RS
	movf	source,w
	call	LCDHEXA		;TX_RS
	movlw	'\n'		; LF
	call	TX_RS		
	movlw	'\r'		; CR
	call	TX_RS
	return

LCDHEXA				; affichage en Hexa d'un caractre
	movwf	HEXA_Temp1		; sauvegarde
	movlw	D'2'
	movwf	HEXA_cpt1
LCDHEXA0			; conversion 0-9 -> H'30'-H'39' ; A-F -> H'41'-H'46' 
	swapf	HEXA_Temp1,f		; Poids fort en 1er
	movf	HEXA_Temp1,w
	andlw	H'0F'		; 0000xxxx
	movwf	HEXA_TEMP
	movlw	D'10'		;
	subwf	HEXA_TEMP,w	; LCD_TEMP - 9
	btfss	STATUS,C
	goto	LCDHEXA2
	movlw	D'7'		
	addwf	HEXA_TEMP,f	; LCD_TEMP + 8
LCDHEXA2
	movlw	H'30'
	addwf	HEXA_TEMP,f	; LCD_TEMP + H'30'

	movf	HEXA_TEMP,w	; caractre  afficher
	call	TX_RS

	decfsz	HEXA_cpt1,f
	goto	LCDHEXA0
	movlw	' '		; espace H'20'
	call	TX_RS
	return


TEST_232_Vp
	movlw	'V'
	call	TX_RS
	movlw	'+'
	call	TX_RS
	ESP
	return

TEST_232_Vm
	movlw	'V'
	call	TX_RS
	movlw	'-'
	call	TX_RS
	ESP
	return

;*********************************************************************
;                      PROGRAMME PRINCIPAL                           *
;*********************************************************************
Clavier
	btfsc	touche,tFADER	; si mode FADER
	goto	ClavFADER	; mode FADER

	btfsc	touche,tVOLIN	; si mode VOLIN
	goto	ClavVOLIN

	btfss	Mute		; * Mute
	goto	ClavMute
		
	btfss	UP		; * Vol+
	goto	ClavUP
	btfss	DOWN		; * Vol-
	goto	ClavDOWN
	btfss	Sp		; * source+
	goto	ClavSp
	btfss	Sm		; * source-
	goto	ClavSm

	btfsc	touche,tMute	; si mode Mute	
	goto	ClavMuteFIN	; ne pas regler le volume

	btfss	touche,tAUCUNE	; si derniere touche != AUCUNE
	clrf	CPTtouche	; RAZ compteur de touche
	incf	CPTtouche,f
	clrf	touche		; RAZ flag de la derniere touche
	bsf	touche,tAUCUNE	; flag AUCUNE touche
	call	TEMPO_2		; tempo 32ms
	btfsc	CPTtouche,4	; si 32ms*16 = 512ms
	call	PREAMP		;#### regler toutes les 32ms*5=160ms
	goto	Clavier		; * AUCUNE TOUCHE

ClavMute
	btfss	touche,tMute	; si derniere touche != touche
	clrf	CPTtouche	; -> RAZ touche
	incf	CPTtouche,f	; CPTtouche++
	btfsc	CPTtouche,5	; si 96ms*32 = 3s
	goto	ClavMODE	; 	Mode FADER/VOLIN
	clrf	touche
	bsf	touche,tMute	; flag MUTE
ClavMuteFIN
	call	TEMPO		; 96ms
	call	PREAMP_MUTE	
	goto	ClavFIN		;;####TEMPORAIRE

ClavUP
	btfss	touche,tUP	; si derniere touche != touche
	clrf	CPTtouche	; -> RAZ CPTtouche
	incf	CPTtouche,f	; CPTtouche++
	btfss	CPTtouche,2	; si <4*tempo (=98ms*4)
	call	TEMPO		; 	tempo 98ms
	btfsc	CPTtouche,2	; sinon
	call	TEMPO_2		; 	tempo  32ms
	decf	volume,f
	decf	volumeAV,f
	;call	TEMPO_2
	clrf	touche
	bsf	touche,tUP
	goto	ClavVOLUME

ClavDOWN
	btfss	touche,tDOWN	; si derniere touche != touche
	clrf	CPTtouche	; -> RAZ CPTtouche
	incf	CPTtouche,f	; CPTtouche++
	btfss	CPTtouche,2	; si <4*tempo (=98ms*4)
	call	TEMPO		; 	tempo 98ms
	btfsc	CPTtouche,2	; sinon
	call	TEMPO_2		; 	tempo  32ms
	incf	volume,f
	incf	volumeAV,f
	clrf	touche
	bsf	touche,tDOWN
	goto	ClavVOLUME

ClavSp
	clrf	CPTtouche	; -> RAZ touche
	;;call	SOURCE232;;;;;####
	;ECR_PARAM volume, volIN, volumeAV, source, tmpEE1,tmpEE2;tmp, data1 ;;#debug
	call	ECR_PARAMETRES
	call	SOURCE232;;;;####
	decf	source,f
	clrf	touche
	bsf	touche,tSp
	goto	ClavSOURCE

ClavSm
	clrf	CPTtouche	; -> RAZ touche
	;;call	SOURCE232;;;;#####
	;ECR_PARAM volume, volIN, volumeAV, source, tmpEE1,tmpEE2;tmp, data1 ;;#debug
	call	ECR_PARAMETRES
	call	SOURCE232;;;;;;###
	incf	source,f
	clrf	touche
	bsf	touche,tSm
	goto	ClavSOURCE

ClavMODE			; mode FADER ou VolIN
	btfss	DOWN		; si * Vol-  -> FADER
	goto	ClavVOLIN	; sinon		VOLIN
	goto	ClavFADER

ClavVOLIN
	btfss	touche,tVOLIN	; si derniere touche != touche
	clrf	CPTtouche	; -> RAZ touche
	btfsc	Mute		; touche Mute pour sortir du mode FADER
	goto	ClavVOLIN2	; -> pas de MUTE : on continue
	incf	CPTtouche,f	; CPTtouche++
	btfsc	CPTtouche,5	; 32ms*32 = 3s pour sortir mode FADER
	goto	ClavMute	
ClavVOLIN2
	btfss	UP		; * Vol+
	decf	volIN,f
	btfss	DOWN		; * Vol-
	incf	volIN,f
	call	TEMPO		; 98ms
	clrf	touche
	bsf	touche,tVOLIN	; flag VOLIN
	movf	volIN,w		; test VolIN au max. 
	btfsc	STATUS,Z	; 	Si volume==0 (-0dB)
	incf	volIN,f		; 	Incrmente VolIN (pour eviter MUTE)
	movlw	B'01111111'	; test VolIN au min
	subwf	volIN,w		; 	B'01111111' - volIN -> w
	btfsc	STATUS,Z	;	Si VolIN==B'01111111'
	decf	volIN,f		;
	call	PREAMP		; reglage volume + attenuation
	goto	ClavFIN
	
	
ClavFADER
	btfss	touche,tFADER	; si derniere touche != touche
	clrf	CPTtouche	; -> RAZ touche
	btfsc	Mute		; touche Mute pour sortir du mode FADER
	goto	ClavFADER2	; -> pas de MUTE : on continue
	incf	CPTtouche,f	; CPTtouche++
	btfsc	CPTtouche,5	; 32ms*32 = 3s pour sortir mode FADER
	goto	ClavMute	
ClavFADER2
	btfss	UP		; * Vol+
	decf	volume,f
	btfss	DOWN		; * Vol-
	incf	volume,f
	btfss	Sp		; * source+
	decf	volumeAV,f	
	btfss	Sm		; * source-
	incf	volumeAV,f
	call	TEMPO		; 98ms
	clrf	touche
	bsf	touche,tFADER	; flag FADER
	;;goto	ClavVOLUME

;ClavVOLUME
;	movf	volume,w	; test Vol au max. 
;	btfsc	STATUS,Z	; 	Si volume==0 (-0dB)
;	incf	volume,f	; 	Incrmente Vol (pour eviter MUTE)
;	movf	volumeAV,w	; test VolAV au max.
;	btfsc	STATUS,Z	; 	Si volumeAV==0 (-0dB)
;	incf	volumeAV,f	; 	Incrmente VolAV (pour eviter MUTE)
;	movlw	B'01111111'	; test Vol au min
;	subwf	volume,w	; 	B'01111111' - volume -> w
;	btfsc	STATUS,Z	;	Si Vol==B'01111111'
;	decf	volume,f	;
;	movlw	B'01111111'	; test VolAV au min
;	subwf	volumeAV,w	; 	B'01111111' - volume -> w
;	btfsc	STATUS,Z	;	Si Vol==B'01111111'
;	decf	volumeAV,f	;

;	call	PREAMP		; reglage volume + attenuation
;	goto	ClavFIN
	
ClavVOLUME
	movf	volume,w	; test Vol au max. 
	btfss	STATUS,Z	; 	Si volume==0 (-0dB)
	goto	ClavVOLUME1
	incf	volume,f	; 	Incrmente Vol (pour eviter MUTE)
	incf	volumeAV,f	;	pb FADER
ClavVOLUME1
	movf	volumeAV,w	; test VolAV au max.
	btfss	STATUS,Z	; 	Si volumeAV==0 (-0dB)
	goto	ClavVOLUME2
	incf	volumeAV,f	; 	Incrmente VolAV (pour eviter MUTE)
	incf	volume,f	;	pb FADER
ClavVOLUME2
	movlw	B'01111111'	; test Vol au min
	subwf	volume,w	; 	B'01111111' - volume -> w
	btfss	STATUS,Z	;	Si Vol==B'01111111'
	goto	ClavVOLUME3
	decf	volume,f	;
	decf	volumeAV,f	; 	pb FADER
ClavVOLUME3
	movlw	B'01111111'	; test VolAV au min
	subwf	volumeAV,w	; 	B'01111111' - volume -> w
	btfss	STATUS,Z	;	Si Vol==B'01111111'
	goto	ClavVOLUME4
	decf	volumeAV,f	;
	decf	volume,f	;	pb FADER
ClavVOLUME4
	call	PREAMP		; reglage volume + attenuation
	goto	ClavFIN	

ClavSOURCE
	;call	PREAMP_MUTE
	;ECR_PARAM volume, volIN, volumeAV, source, tmp, data1
	movlw	SourceMAX	; charge 5 dans W
	subwf	source,w	; w-source->W : 5-source->W
	btfsc	STATUS,Z	; si source == 5
	clrf	source		;	source = 0
		
	movlw	h'ff'		; charge 255 dans W
	subwf	source,w	; w-source->W : 255-source->W
	btfss	STATUS,Z	; si source != 255
	goto	ClavSOURCE2
	movlw	SourceMAX-1	;	source = 4
	movwf	source		;	source = 4
ClavSOURCE2
	call	SWITCH_SOURCE
	call	SOURCE232;;;;;;;####
	call	PREAMP		; de SWITCH_SOURCE
	call	TEMPO 		;###??
	call	TEMPO		;##???
	goto	ClavFIN

ClavFIN 			; fin du traitement clavier
	goto	Clavier



start
	call 	TEST_232
	goto	Clavier ;#########DEBUG#########
start2	

;	btfss	Sp		; si source+
;	call	TEST_232_Source
;	btfss	Sm
;	call	TEST_232_Source

	goto 	start2		; boucler

	END 			; directive fin de programme

